/*******************************************************************************************
 *
 *   raylib-extras [ImGui] example - Docking example
 *
 *	This is an example of using the ImGui docking features that are part of docking branch
 *	You must replace the default imgui with the code from the docking branch for this to work
 *	https://github.com/ocornut/imgui/tree/docking
 *
 *   Copyright (c) 2024 Jeffery Myers
 *
 ********************************************************************************************/

#define IMGUI_DEFINE_MATH_OPERATORS
#include "course.hpp"
#include "raylib.h"
#include "raymath.h"
#include <optional>
#include <unordered_map>
#include <vector>

#include "imgui.h"
#include "imgui/misc/cpp/imgui_stdlib.h"
#include "rlImGui/rlImGui.h"

#include "testcpp.hpp"

#include "nlohmann/json.hpp"
#include "stdio.h"

#include <fstream>
#include <iostream>
#include <string>

extern "C" {
#include "testc.h"
}

using json = nlohmann::json;

class RayWindow {
  public:
    bool Open = false;

    RenderTexture ViewTexture;

    virtual void Setup() = 0;
    virtual void Shutdown() = 0;
    virtual void Show() = 0;
    virtual void Update() = 0;

    bool Focused = false;

    Rectangle ContentRect = {0};
};

void ApplyJsonToCourse(Course *c, json j) {
    c->name = j.at("name");
    // printf("Parsing: %s\n", c->name.c_str());
    if (j.at("desc").is_null()) {
    } else {
        c->desc = j.at("desc");
    }

    if (j.at("prereqs_obj").at("reqs_list").is_null()) {
    } else {
        c->prereq = j.at("prereqs_obj").at("reqs_list");
    }
}

std::optional<Course> GetCourseFromNum(std::vector<Course> course, std::string num) {
    for (int i = 0; i < course.size(); i++) {
        if (course[i].number == num) {
            return course[i];
        }
    }
    return std::nullopt;
}

class RayWindowExample {
  public:
    RenderTexture ViewTexture;
    bool Open = true;
    bool Focused = false;
    bool Resized = false;
    ImVec2 PastSize;
    RayWindowExample() {
        ViewTexture = LoadRenderTexture(GetScreenWidth(), GetScreenWidth());
        // ViewTexture = LoadRenderTexture(256, 256);
        printf("window w,h: %d, %d \n", ViewTexture.texture.width, ViewTexture.texture.height);
        PastSize = ImVec2{256, 256};
        Resized = false;
    }
    ~RayWindowExample() {
        UnloadRenderTexture(ViewTexture);
    }
    void Show() {
        ImGui::PushStyleVar(ImGuiStyleVar_WindowPadding, ImVec2(0, 0));
        ImGui::SetNextWindowSize(ImVec2(256, 275), ImGuiCond_FirstUseEver);
        if (ImGui::Begin("Raylib in ImGUI", &Open, ImGuiWindowFlags_NoScrollbar)) {
            Focused = ImGui::IsWindowFocused(ImGuiFocusedFlags_ChildWindows);
            ImVec2 DaZone = ImGui::GetContentRegionAvail();
            rlImGuiImageRenderTexture(&ViewTexture);
            // rlImGuiImage(&ViewTexture.texture);
            if (PastSize != DaZone) {
                Resized = true;
            } else {
                Resized = false;
            }
            PastSize = DaZone;
        }
        ImGui::End();
        ImGui::PopStyleVar();
    }
    void Update() {
        if (!Open) {
            return;
        }
        if (Resized) {
            UnloadRenderTexture(ViewTexture);
            ViewTexture = LoadRenderTexture(PastSize.x, PastSize.y);
        }

        BeginTextureMode(ViewTexture);
        ClearBackground(SKYBLUE);
        DrawCircle(0, 0, 35, RED);
        //  DrawPixel(0,0, BLUE);
        //  DrawRectangleLines(0,0,PastSize.x,PastSize.y, RED);
        Rectangle rec = Rectangle{0, 0, PastSize.x, PastSize.y};
        // DrawRectangleLinesEx(rec, 2, RED);
        EndTextureMode();
    }
};

int main(int argc, char *argv[]) {
    // Initialization
    //--------------------------------------------------------------------------------------
    int screenWidth = 1280;
    int screenHeight = 800;

    // SetConfigFlags(FLAG_MSAA_4X_HINT | FLAG_WINDOW_RESIZABLE);
    SetConfigFlags(FLAG_MSAA_4X_HINT | FLAG_VSYNC_HINT | FLAG_WINDOW_RESIZABLE);
    InitWindow(screenWidth, screenHeight, "raylib-Extras [ImGui] example - Docking");
    SetTargetFPS(120);
    rlImGuiSetup(true);
    ImGui::GetStyle().AntiAliasedLinesUseTex = false;

    TestC();
    TestCPPFunc();

    bool run = true;

    bool showDemoWindow = false;

    // if the linked ImGui has docking, enable it.
    // this will only be true if you use the docking branch of ImGui.
#ifdef IMGUI_HAS_DOCK
    ImGui::GetIO().ConfigFlags |= ImGuiConfigFlags_DockingEnable;
#endif

    std::ifstream f("spring2025.json");
    json course_info = json::parse(f);

    // for (json::iterator it = course_info.begin(); it != course_info.end(); ++it) {
    //     std::cout << *it << std::endl;
    // }

    json list = course_info.at("courses");

    std::unordered_map<std::string, Course> all_courses;

    for (json::iterator it = list.begin(); it != list.end(); ++it) {
        Course test = Course(it.key());
        ApplyJsonToCourse(&test, it.value());
        // printf("%s\n", test.name.c_str());
        all_courses.insert(std::make_pair(it.key(), test));
    }

    CourseManager courseman(all_courses);

    printf("The rot consumed the json course data!!\n");

    std::vector<CourseWindow> course_windows;
    std::string new_course_text_box_string;
    // Main game loop
    while (!WindowShouldClose() &&
           run) // Detect window close button or ESC key, or a quit from the menu
    {

        BeginDrawing();
        ClearBackground(DARKGRAY);

        // start ImGui content
        rlImGuiBegin();

        // if you want windows to dock to the viewport, call this.
#ifdef IMGUI_HAS_DOCK
        ImGui::DockSpaceOverViewport();
#endif

        // show a simple menu bar
        if (ImGui::BeginMainMenuBar()) {
            if (ImGui::BeginMenu("File")) {
#ifdef IMGUI_HAS_DOCK
                if (ImGui::MenuItem("Enable Docking")) {
                    ImGui::GetIO().ConfigFlags |= ImGuiConfigFlags_DockingEnable;
                }
                if (ImGui::MenuItem("Disable Docking")) {
                    ImGui::GetIO().ConfigFlags &= ~ImGuiConfigFlags_DockingEnable;
                }
#endif
                if (ImGui::MenuItem("Quit"))
                    run = false;

                ImGui::EndMenu();
            }

            if (ImGui::BeginMenu("Window")) {
                if (ImGui::MenuItem("Demo Window", nullptr, showDemoWindow))
                    showDemoWindow = !showDemoWindow;

                ImGui::EndMenu();
            }
            ImGui::EndMainMenuBar();
        }

        // show some windows

        if (showDemoWindow)
            ImGui::ShowDemoWindow(&showDemoWindow);

        if (ImGui::Begin("Test Window")) {
            ImGui::TextUnformatted("Add Course To Display");
            ImGui::InputText("erm", &new_course_text_box_string);
            if (ImGui::Button("Pog!!!!!!")) {
                courseman.AddByNum(new_course_text_box_string);
            }
        }
        ImGui::End();

        courseman.Draw();

        // end ImGui Content
        rlImGuiEnd();

        EndDrawing();
        //----------------------------------------------------------------------------------
    }
    rlImGuiShutdown();

    // De-Initialization
    //--------------------------------------------------------------------------------------
    CloseWindow(); // Close window and OpenGL context
    //--------------------------------------------------------------------------------------

    return 0;
}
